package mqtt_api

import (
	"git.sxidc.com/service-supports/fslog"
	"runtime/debug"
	"slices"
	"time"
)

type MqttApi struct {
	options Options
	router  *Router
}

func New(opts ...Option) (*MqttApi, error) {
	options := new(Options)

	for _, opt := range opts {
		opt(options)
	}

	router := NewRouter(options.topicPrefix, []Handler{
		func(c *Context) {
			start := time.Now()
			c.Next()

			if slices.Contains(options.logSkipPaths, c.Topic) {
				return
			}

			end := time.Now()
			fslog.Info("| %d | %s |",
				end.Sub(start),
				c.Topic,
			)
		},
		func(c *Context) {
			defer func() {
				err := recover()
				if err != nil {
					fslog.Error("%s", debug.Stack())
				}
			}()

			c.Next()
		},
	})

	return &MqttApi{
		options: *options,
		router:  router,
	}, nil
}

func Destroy(api *MqttApi) {
	DestroyRouter(api.router)
	api.router = nil
}

// Start 运行Api
// 参数: 无
// 返回值:
// - 错误
func (api *MqttApi) Start() error {
	err := api.router.Start(api.options.mqttOptions)
	if err != nil {
		return err
	}

	return nil
}

// Finish 终止Api
// 参数: 无
// 返回值: 无
func (api *MqttApi) Finish() {
	if api.router == nil {
		return
	}

	api.router.Finish()
	api.router = nil
}

// Options 获取Api的选项
// 参数: 无
// 返回值:
// - Api的选项
func (api *MqttApi) Options() Options {
	return api.options
}

// Router 获取Api的路由
// 参数:
// - 消息组
// 返回值:
// - Api的路由
func (api *MqttApi) Router() *Router {
	return api.router
}
